import * as React from 'react';
import { ButtonGroup } from "./ButtonGroup";
import { Button } from "./Button";
import { ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { IgrButtonGroupButtonCollection } from './igr-button-group-button-collection';
import { CollectionAdapter } from "igniteui-react-core";
import { NotifyCollectionChangedAction } from "igniteui-react-core";
import { IgrButtonGroupSelectionChangedEventArgs } from './igr-button-group-selection-changed-event-args';
import { ContentChildrenManager } from "igniteui-react-core";
import { ButtonGroupOrientation_$type } from './ButtonGroupOrientation';
import { ControlDisplayDensity_$type } from "igniteui-react-core";
import { BaseControlTheme_$type } from "igniteui-react-core";
export class IgrButtonGroup extends React.Component {
    constructor(props) {
        super(props);
        this._wrapper = null;
        this.contentButtons = [];
        /**
        * The style mappings actually present in the treemap. Do not directly modify this array.
        * This array's contents can be modified by causing React to reproject the child content.
        * Or adding and removing ranges from the manual ranges collection on the ranges property.
        */
        this.actualButtons = [];
        this._buttons = null;
        this._buttonsAdapter = null;
        this._implementation = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._selectionChanged = null;
        this._selectionChanged_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._contentChildrenManager = new ContentChildrenManager((ch) => ch.props.name || ch.key || ch.props.id, (ch) => ch.name || ch.key || ch.id, () => this._updateContentChildren());
        this._implementation = this.createImplementation();
        this._implementation.externalObject = this;
        this._getMainRef = this._getMainRef.bind(this);
        var container;
        if (document) {
            container = document.createElement("div");
        }
        var root;
        root = container;
        if (container != null) {
            root = container;
        }
        this.requestRender = this.requestRender.bind(this);
        this._portalManager = new PortalManager("buttonContent", this.requestRender);
        var ren = new ReactRenderer(root, document, true, {}, this._portalManager);
        this._wrapper = ren;
        this._container = this._wrapper.rootWrapper;
        this._container.setStyleProperty("display", "inline-block");
        var buttonGroup = this.i;
        this._buttonGroup = buttonGroup;
        this._buttonsAdapter = new CollectionAdapter(this.contentButtons, this.i.buttons, this.actualButtons, (c) => c.i, (i) => {
            if (this._container && this._container.getNativeElement().parentElement) {
                i._styling(this._container.getNativeElement(), this, this);
            }
        }, (i) => { });
        buttonGroup.provideContainer(ren);
        if (props) {
            this.initializeProperties();
        }
    }
    _getMainRef(ref) {
        this._elRef = ref;
    }
    _updateContentChildren() {
        this.contentButtons.length = 0;
        let contentChildrenActual = this._contentChildrenManager.contentChildrenActual;
        for (let i = 0; i < contentChildrenActual.length; i++) {
            if ((Button.$type).isAssignableFrom(contentChildrenActual[i].i.$type)) {
                this.contentButtons.push(contentChildrenActual[i]);
            }
        }
        if (this._buttonsAdapter !== null) {
            this._buttonsAdapter.notifyContentChanged();
        }
    }
    /**
     * A collection of manually added style mappings for the treemap.
    */
    get buttons() {
        if (this._buttons === null) {
            let coll = new IgrButtonGroupButtonCollection();
            let inner = coll._innerColl;
            inner.addListener((sender, e) => {
                switch (e.action) {
                    case NotifyCollectionChangedAction.Add:
                        this._buttonsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        this._buttonsAdapter.removeManualItemAt(e.oldStartingIndex);
                        break;
                    case NotifyCollectionChangedAction.Replace:
                        this._buttonsAdapter.removeManualItemAt(e.oldStartingIndex);
                        this._buttonsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Reset:
                        this._buttonsAdapter.clearManualItems();
                        break;
                }
            });
            this._buttons = coll;
        }
        return this._buttons;
    }
    render() {
        let children = this._contentChildrenManager.getChildren(this.props.children);
        this._portalManager.onRender(children);
        let div = React.createElement("div", {
            className: "ig-button-group igr-button-group",
            ref: this._getMainRef,
            children: children
        });
        return div;
    }
    requestRender() {
        if (this._initialized) {
            this.setState({});
        }
    }
    //private _getLabelPortal(element: DomWrapper, portalCallback: (portal: DomPortal) => void): void {
    //    this._portalManager.getPortal(element, "TemplateContent", portalCallback);
    //}
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        this._buttonGroup.destroy();
        this._wrapper.destroy();
    }
    componentDidMount() {
        this._elRef.appendChild(this._container.getNativeElement());
        this.initializeContent();
    }
    initializeContent() {
        this._styling(this._container.getNativeElement(), this);
        this.updateStyle();
    }
    createImplementation() {
        return new ButtonGroup();
    }
    get i() {
        return this._implementation;
    }
    /**
 * Gets or sets the base built in theme to use for the button group.
*/
    get baseTheme() {
        return this.i.g;
    }
    set baseTheme(v) {
        this.i.g = ensureEnum(BaseControlTheme_$type, v);
    }
    /**
     * Gets or sets the display density to use for the button group.
    */
    get density() {
        return this.i.k;
    }
    set density(v) {
        this.i.k = ensureEnum(ControlDisplayDensity_$type, v);
    }
    /**
     * Gets the actual display density to use for the button group.
    */
    get actualDensity() {
        return this.i.j;
    }
    set actualDensity(v) {
        this.i.j = ensureEnum(ControlDisplayDensity_$type, v);
    }
    /**
     * Gets the currently selected button indices.
    */
    get selectedIndices() {
        return this.i.d;
    }
    set selectedIndices(v) {
        this.i.d = v;
    }
    /**
     * Gets the actual item corner radius for the appropriate corners.
    */
    get actualItemCornerRadius() {
        return this.i.ae;
    }
    set actualItemCornerRadius(v) {
        this.i.ae = +v;
    }
    /**
     * Gets the actual item background color.
    */
    get actualItemBackgroundColor() {
        return brushToString(this.i.cx);
    }
    set actualItemBackgroundColor(v) {
        this.i.cx = stringToBrush(v);
    }
    /**
     * Gets the actual item background color.
    */
    get actualItemDisabledBackgroundColor() {
        return brushToString(this.i.cz);
    }
    set actualItemDisabledBackgroundColor(v) {
        this.i.cz = stringToBrush(v);
    }
    /**
     * Gets the actual item text color.
    */
    get actualItemTextColor() {
        return brushToString(this.i.c4);
    }
    set actualItemTextColor(v) {
        this.i.c4 = stringToBrush(v);
    }
    /**
     * Gets the actual item text color.
    */
    get actualItemDisabledTextColor() {
        return brushToString(this.i.c1);
    }
    set actualItemDisabledTextColor(v) {
        this.i.c1 = stringToBrush(v);
    }
    /**
     * Gets the actual item border color.
    */
    get actualItemBorderColor() {
        return brushToString(this.i.cy);
    }
    set actualItemBorderColor(v) {
        this.i.cy = stringToBrush(v);
    }
    /**
     * Gets the actual item border color.
    */
    get actualItemDisabledBorderColor() {
        return brushToString(this.i.c0);
    }
    set actualItemDisabledBorderColor(v) {
        this.i.c0 = stringToBrush(v);
    }
    /**
     * Gets the actual item border width.
    */
    get actualItemBorderWidth() {
        return this.i.ad;
    }
    set actualItemBorderWidth(v) {
        this.i.ad = +v;
    }
    /**
     * Gets the actual selected item background color.
    */
    get actualSelectedItemBackgroundColor() {
        return brushToString(this.i.c5);
    }
    set actualSelectedItemBackgroundColor(v) {
        this.i.c5 = stringToBrush(v);
    }
    /**
     * Gets the stroke actual selected item text color.
    */
    get actualSelectedItemTextColor() {
        return brushToString(this.i.c8);
    }
    set actualSelectedItemTextColor(v) {
        this.i.c8 = stringToBrush(v);
    }
    /**
     * Gets the actual item hovered background color.
    */
    get actualItemHoverBackgroundColor() {
        return brushToString(this.i.c2);
    }
    set actualItemHoverBackgroundColor(v) {
        this.i.c2 = stringToBrush(v);
    }
    /**
     * Gets the actual item text color.
    */
    get actualItemHoverTextColor() {
        return brushToString(this.i.c3);
    }
    set actualItemHoverTextColor(v) {
        this.i.c3 = stringToBrush(v);
    }
    /**
     * Gets the actual selected item hovered background color.
    */
    get actualSelectedItemHoverBackgroundColor() {
        return brushToString(this.i.c6);
    }
    set actualSelectedItemHoverBackgroundColor(v) {
        this.i.c6 = stringToBrush(v);
    }
    /**
     * Gets the actual selected item hovered text color.
    */
    get actualSelectedItemHoverTextColor() {
        return brushToString(this.i.c7);
    }
    set actualSelectedItemHoverTextColor(v) {
        this.i.c7 = stringToBrush(v);
    }
    /**
     * Gets or sets the corner radius to use for the appropriate corners for the item buttons.
    */
    get itemCornerRadius() {
        return this.i.am;
    }
    set itemCornerRadius(v) {
        this.i.am = +v;
    }
    /**
     * Gets or sets the color to use the background of the buttons in the group.
    */
    get itemBackgroundColor() {
        return brushToString(this.i.ea);
    }
    set itemBackgroundColor(v) {
        this.i.ea = stringToBrush(v);
    }
    /**
     * Gets or sets the disabled color to use the background of the buttons in the group.
    */
    get itemDisabledBackgroundColor() {
        return brushToString(this.i.ec);
    }
    set itemDisabledBackgroundColor(v) {
        this.i.ec = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use the text of the buttons in the group.
    */
    get itemTextColor() {
        return brushToString(this.i.eh);
    }
    set itemTextColor(v) {
        this.i.eh = stringToBrush(v);
    }
    /**
     * Gets or sets the disabled color to use the text of the buttons in the group.
    */
    get itemDisabledTextColor() {
        return brushToString(this.i.ee);
    }
    set itemDisabledTextColor(v) {
        this.i.ee = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use the hovered background of the buttons in the group.
    */
    get itemHoverBackgroundColor() {
        return brushToString(this.i.ef);
    }
    set itemHoverBackgroundColor(v) {
        this.i.ef = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use the border of the buttons in the group.
    */
    get itemBorderColor() {
        return brushToString(this.i.eb);
    }
    set itemBorderColor(v) {
        this.i.eb = stringToBrush(v);
    }
    /**
     * Gets or sets the disabled color to use the border of the buttons in the group.
    */
    get itemDisabledBorderColor() {
        return brushToString(this.i.ed);
    }
    set itemDisabledBorderColor(v) {
        this.i.ed = stringToBrush(v);
    }
    /**
     * Gets or sets the border width to use for the buttons in the group.
    */
    get itemBorderWidth() {
        return this.i.al;
    }
    set itemBorderWidth(v) {
        this.i.al = +v;
    }
    /**
     * Gets or sets the color to use the hovered text of the buttons in the group.
    */
    get itemHoverTextColor() {
        return brushToString(this.i.eg);
    }
    set itemHoverTextColor(v) {
        this.i.eg = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use the background of the buttons in the group that are selected.
    */
    get selectedItemBackgroundColor() {
        return brushToString(this.i.ei);
    }
    set selectedItemBackgroundColor(v) {
        this.i.ei = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use the text of the buttons in the group that are selected.
    */
    get selectedItemTextColor() {
        return brushToString(this.i.el);
    }
    set selectedItemTextColor(v) {
        this.i.el = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use the hovered background of the buttons in the group that are selected.
    */
    get selectedItemHoverBackgroundColor() {
        return brushToString(this.i.ej);
    }
    set selectedItemHoverBackgroundColor(v) {
        this.i.ej = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use the hovered text of the buttons in the group that are selected.
    */
    get selectedItemHoverTextColor() {
        return brushToString(this.i.ek);
    }
    set selectedItemHoverTextColor(v) {
        this.i.ek = stringToBrush(v);
    }
    /**
     * Gets or sets the oreientation to use the button group.
    */
    get orientation() {
        return this.i.o;
    }
    set orientation(v) {
        this.i.o = ensureEnum(ButtonGroupOrientation_$type, v);
    }
    /**
     * Gets or sets the id to use for the checkbox.
    */
    get id() {
        return this.i.a7;
    }
    set id(v) {
        this.i.a7 = v;
    }
    /**
     * Gets or sets whether the checkbox is disabled.
    */
    get disabled() {
        return this.i.disabled;
    }
    set disabled(v) {
        this.i.disabled = ensureBool(v);
    }
    /**
     * Gets or sets whether the group supports multi-select.
    */
    get isMultiSelect() {
        return this.i.w;
    }
    set isMultiSelect(v) {
        this.i.w = ensureBool(v);
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ButtonGroup");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    onDetachedFromUI() {
        this.i.onDetachedFromUI();
    }
    onAttachedToUI() {
        this.i.onAttachedToUI();
    }
    get selectionChanged() {
        return this._selectionChanged;
    }
    set selectionChanged(ev) {
        if (this._selectionChanged_wrapped !== null) {
            this.i.selectionChanged = delegateRemove(this.i.selectionChanged, this._selectionChanged_wrapped);
            this._selectionChanged_wrapped = null;
            this._selectionChanged = null;
        }
        this._selectionChanged = ev;
        this._selectionChanged_wrapped = (o, e) => {
            let outerArgs = new IgrButtonGroupSelectionChangedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeSelectionChanged) {
                this.beforeSelectionChanged(this, outerArgs);
            }
            if (this._selectionChanged) {
                this._selectionChanged(this, outerArgs);
            }
        };
        this.i.selectionChanged = delegateCombine(this.i.selectionChanged, this._selectionChanged_wrapped);
    }
}
//# sourceMappingURL=igr-button-group.js.map
